use axum::{
    extract::Path,
    response::{Html, IntoResponse},
    Extension,
};
use axum_extra::extract::CookieJar;
use tetratto_core::model::{permissions::FinePermission, Error};
use crate::{get_user_from_token, State};
use super::render_error;

/// `/links/{id}`
pub async fn navigate_request(
    jar: CookieJar,
    Path(id): Path<usize>,
    Extension(data): Extension<State>,
) -> impl IntoResponse {
    let data = data.read().await;
    let user = match get_user_from_token!(jar, data.0) {
        Some(ua) => ua,
        None => {
            return Err(Html(
                render_error(Error::NotAllowed, &jar, &data, &None).await,
            ));
        }
    };

    let link = match data.0.get_link_by_id(id).await {
        Ok(x) => x,
        Err(e) => return Err(Html(render_error(e, &jar, &data, &Some(user)).await)),
    };

    let owner = match data.0.get_user_by_id(link.owner).await {
        Ok(x) => x,
        Err(e) => return Err(Html(render_error(e, &jar, &data, &Some(user)).await)),
    };

    if owner.permissions.check(FinePermission::SUPPORTER) {
        if let Err(e) = data.0.incr_link_clicks(link.id).await {
            return Err(Html(render_error(e, &jar, &data, &Some(user)).await));
        }
    }

    Ok(Html(format!(
        "<!doctype html /><html><head><meta http-equiv=\"refresh\" content=\"0; url={}\" /></head><body>Navigating...</body></html>",
        link.href
    )))
}
